#define G_LOG_DOMAIN "boilerplate-plugins-engine"

#include "config.h"

#include "boilerplate-plugins-engine.h"
#include "boilerplate-application.h"

#include <glib.h>
#include <libpeas/peas.h>

typedef struct _BoilerplatePluginsEnginePrivate
{
    PeasExtensionSet *extension_set;
    GPtrArray *paths;

} BoilerplatePluginsEnginePrivate;

struct _BoilerplatePluginsEngine
{
    PeasEngine parent_instance;
};

/**
 * SECTION:boilerplate-plugins-engine
 * @title: BoilerplatePluginsEngine
 * @short_description: Wrapper object for initializing plugins
 * @include: boilerplate-plugins-engine.h
 * @see_also: #PeasEngine
 *
 * #BoilerplatePluginsEngine is a wrapper object for initializing the #PeasEngine.
 * It also sets search paths for plugins directory.
 **/

G_DEFINE_TYPE_WITH_PRIVATE (BoilerplatePluginsEngine, boilerplate_plugins_engine, PEAS_TYPE_ENGINE)

/* Singleton instance */
BoilerplatePluginsEngine *default_engine = NULL;

enum {
    PROP_0,
    PROP_PATH,
    LAST_PROP
};

static void
on_extension_added (PeasExtensionSet *set,
                    PeasPluginInfo   *info,
                    PeasExtension    *extension,
                    gpointer          user_data)
{
	BoilerplatePluginsEngine *self = BOILERPLATE_PLUGINS_ENGINE (user_data);

    g_assert (BOILERPLATE_IS_PLUGINS_ENGINE (self));

    peas_activatable_activate (PEAS_ACTIVATABLE (extension));
}

static void
on_extension_removed (PeasExtensionSet *set,
                      PeasPluginInfo   *info,
                      PeasExtension    *extension,
                      gpointer          user_data)
{
	BoilerplatePluginsEngine *self = BOILERPLATE_PLUGINS_ENGINE (user_data);

    g_assert (BOILERPLATE_IS_PLUGINS_ENGINE (self));

    peas_activatable_deactivate (PEAS_ACTIVATABLE (extension));
}

/**
 * boilerplate_plugins_engine_get_property:
 * @object: The current gObject
 * @prop_id: The property id
 * @value: The value to get
 * @pspec: A g_param_spec_* ()
 *
 * Get property in #BoilerplatePluginsEngine.
 */
static void
boilerplate_plugins_engine_get_property (GObject    *object,
                                         guint       prop_id,
                                         GValue     *value,
                                         GParamSpec *pspec)
{
    BoilerplatePluginsEngine *self = BOILERPLATE_PLUGINS_ENGINE (object);
    BoilerplatePluginsEnginePrivate *priv = boilerplate_plugins_engine_get_instance_private (self);

    switch (prop_id)
    {
        case PROP_PATH:
            g_value_set_object (value,
                                priv->paths);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    }
}

/**
 * boilerplate_plugins_engine_set_property:
 * @object: The current gObject
 * @prop_id: The property id
 * @value: The value to set
 * @pspec: A g_param_spec_* ()
 *
 * Set property in #BoilerplatePluginsEngine.
 */
static void
boilerplate_plugins_engine_set_property (GObject      *object,
                                         guint         prop_id,
                                         const GValue *value,
                                         GParamSpec   *pspec)
{
    BoilerplatePluginsEngine *self = BOILERPLATE_PLUGINS_ENGINE (object);
    BoilerplatePluginsEnginePrivate *priv = boilerplate_plugins_engine_get_instance_private (self);

    switch (prop_id)
    {
        case PROP_PATH:
            peas_engine_add_search_path (PEAS_ENGINE (self),
                                         g_value_get_string (value),
                                         g_value_get_string (value));

            peas_engine_rescan_plugins (PEAS_ENGINE (self));

            if (priv->paths == NULL)
            {
                priv->paths = g_ptr_array_new ();
            }

            g_ptr_array_add (priv->paths,
                             g_value_get_pointer (value));
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    }
}

void
boilerplate_plugins_engine_shutdown (BoilerplatePluginsEngine *self)
{
    BoilerplatePluginsEnginePrivate *priv = boilerplate_plugins_engine_get_instance_private (self);

    g_return_if_fail (BOILERPLATE_IS_PLUGINS_ENGINE (self));
    g_return_if_fail (priv->extension_set != NULL);

    g_signal_handlers_disconnect_by_func (priv->extension_set,
                                          G_CALLBACK (on_extension_added),
                                          self);

    g_signal_handlers_disconnect_by_func (priv->extension_set,
                                          G_CALLBACK (on_extension_removed),
                                          self);

    peas_extension_set_call (priv->extension_set,
                             "deactivate");

    g_object_unref (priv->extension_set);

    priv->extension_set = NULL;
}

static void
boilerplate_plugins_engine_dispose (GObject *object)
{
    BoilerplatePluginsEngine *self = BOILERPLATE_PLUGINS_ENGINE (object);
    BoilerplatePluginsEnginePrivate *priv = boilerplate_plugins_engine_get_instance_private (self);

    if (priv->extension_set != NULL)
    {
        boilerplate_plugins_engine_shutdown (self);
    }

    if (priv->paths)
    {
        g_ptr_array_free (priv->paths, TRUE);
        g_object_unref (priv->paths);
    }

    priv->paths = NULL;

    G_OBJECT_CLASS (boilerplate_plugins_engine_parent_class)->dispose (object);
}

static void
boilerplate_plugins_engine_init (BoilerplatePluginsEngine *self)
{
    BoilerplatePluginsEnginePrivate *priv = boilerplate_plugins_engine_get_instance_private (self);

    priv->paths = g_ptr_array_new ();

    peas_engine_enable_loader (PEAS_ENGINE (self),
                               "python3");

    /* Add default plugins dir */
    peas_engine_add_search_path (PEAS_ENGINE (self),
	                             PACKAGE_PLUGIN_DIRECTORY,
                                 PACKAGE_PLUGIN_DIRECTORY);
}

static void
boilerplate_plugins_engine_class_init (BoilerplatePluginsEngineClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->dispose = boilerplate_plugins_engine_dispose;

    object_class->get_property = boilerplate_plugins_engine_get_property;
    object_class->set_property = boilerplate_plugins_engine_set_property;

	/**
	 * BoilerplatePluginsEngine:path:
	 *
	 * The path property contains the a saerch path that #BoilerplatePluginsEngine
     * should search for plugins.
	 */
    g_object_class_install_property (object_class,
                                     PROP_PATH,
                                     g_param_spec_string ("path",
                                                          "Search path",
                                                          "Directory search path for plugins.",
                                                          NULL,
                                                          G_PARAM_READWRITE));
}

/**
 * boilerplate_plugins_engine_new:
 *
 * Creates a new #PeasEngine instance as #BoilerplatePlguinsEngine.
 *
 * Returns: #BoilerplatePlguinsEngine
 */
BoilerplatePluginsEngine *
boilerplate_plugins_engine_new (void)
{
    return g_object_new (BOILERPLATE_TYPE_PLUGINS_ENGINE,
                         NULL);
}

/**
 * boilerplate_plugins_engine_get_default:
 *
 * Gets the default #PeasEngine, if does not exist create a new instance of
 * #BoilerplatePlguinsEngine.
 *
 * Returns: #BoilerplatePlguinsEngine
 */
BoilerplatePluginsEngine *
boilerplate_plugins_engine_get_default (void)
{
    if (!default_engine)
    {
        default_engine = boilerplate_plugins_engine_new ();
    }

    return default_engine;
}

/*
 *******************************************************************************
 * API
 *******************************************************************************
 */

/**
 * boilerplate_plugins_engine_set_path:
 * @self: The #BoilerplatePluginsEngine
 * @user_data: A path that should be used to search for plugins
 *
 * Sets a path #PeasEngine should search for plugins, saves the path in a variable
 * in the the private struct.
 */
void
boilerplate_plugins_engine_set_path (BoilerplatePluginsEngine *self,
                                     gpointer                  user_data)
{
    g_return_if_fail (BOILERPLATE_IS_PLUGINS_ENGINE (self));

    BoilerplatePluginsEnginePrivate *priv = boilerplate_plugins_engine_get_instance_private (self);

    peas_engine_add_search_path (PEAS_ENGINE (self),
                                 user_data,
                                 user_data);

    if (priv->paths == NULL)
    {
        priv->paths = g_ptr_array_new ();
    }

    g_ptr_array_add (priv->paths,
                     user_data);
}

/**
 * boilerplate_plugins_engine_get_path:
 * @self: The #BoilerplatePluginsEngine
 *
 * Gets the stored paths that #PeasEngine uses to search for plugins.
 *
 * Returns: #G_TYPE_ARRAY The array of paths #BoilerplatePluginsEngine is searching.
 */
char **
boilerplate_plugins_engine_get_path (BoilerplatePluginsEngine *self)
{
    g_return_val_if_fail (BOILERPLATE_IS_PLUGINS_ENGINE (self), NULL);

    BoilerplatePluginsEnginePrivate *priv = boilerplate_plugins_engine_get_instance_private (self);

    if (priv->paths == NULL)
    {
        priv->paths = g_ptr_array_new ();
    }

    return (char **) g_ptr_array_free (priv->paths, FALSE);
}
