#define G_LOG_DOMAIN "boilerplate-preference"

#include "config.h"

#include "boilerplate-preferences.h"
#include "boilerplate-window.h"
#include "boilerplate-application.h"

#include <gtk/gtk.h>
#include <glib/gi18n.h>

/**
 * SECTION:boilerplate-preferences
 * @title: BoilerplatePreferences
 * @short_description: A #GtkWindow instance created for #BoilerplateApplication
 * @include: boilerplate-preferences.h
 * @see_also: #GtkWindow
 *
 * The #BoilerplatePreferences is an instance #GtkWindow, and acts as
 * the settings window for the current #GtkApplication.
 **/

typedef struct _BoilerplatePreferencesPrivate
{
    BoilerplateWindow *window;
    GSettings *settings;

    GtkWidget *toolbar;
    GtkWidget *headerbar;
    GtkWidget *content;

    GtkWidget *pages;
    GtkWidget *sidebar;

    GtkWidget *example_button;
    GtkWidget *example_combo;
} BoilerplatePreferencesPrivate;

struct _BoilerplatePreferences
{
    GtkWindow parent_instance;
};

G_DEFINE_TYPE_WITH_PRIVATE (BoilerplatePreferences, boilerplate_preferences, GTK_TYPE_WINDOW)

/*
 * _on_key_press:
 * @widget: The original #GtkWidget that created the #G_CALLBACK
 * @event: The #GdkEventKey or the key that was pressed to start the callback
 * @user_data: User data set when the signal handler was connected, in this
 * instance it should be a #GtkWidget (#BoilerplatePreferences).
 *
 * On a keypress from the user, check for a case in the switch that matchs the
 * keypress. If the keypress exists, run that code and return %TRUE. Otherwise
 * return %FALSE.
 *
 * Returns: #gboolean; %TRUE key press case found and code executed or %FALSE
 * no case found.
 */
static gboolean
_on_key_press (GtkWidget   *widget,
               GdkEventKey *event,
               gpointer     user_data)
{
    switch (event->keyval)
    {
        case GDK_KEY_Escape:
            gtk_widget_destroy (widget);
            return TRUE;

            break;

        default:
            return FALSE;
    }

    return FALSE;
}

/*
 * boilerplate_preferences_add_page:
 * @object: An instance of #BoilerplatePreferences
 *
 * Does any last minute clean-ups when the class ends.
 */
void
boilerplate_preferences_add_page (BoilerplatePreferences *self,
                                  GtkWidget              *page,
                                  const gchar            *name,
                                  const gchar            *title)
{
    g_return_if_fail (BOILERPLATE_IS_PREFERENCES (self));

    BoilerplatePreferencesPrivate *priv = boilerplate_preferences_get_instance_private (BOILERPLATE_PREFERENCES (self));

    gtk_container_add_with_properties (GTK_CONTAINER (priv->pages), GTK_WIDGET (page),
                                       "name", name,
                                       "title", title,
                                       NULL);
}

/*
 * boilerplate_preferences_finalize:
 * @object: An instance of #BoilerplatePreferences
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
boilerplate_preferences_finalize (GObject *object)
{
    BoilerplatePreferencesPrivate *priv = boilerplate_preferences_get_instance_private (BOILERPLATE_PREFERENCES (object));

    g_clear_object (&priv->settings);

    G_OBJECT_CLASS (boilerplate_preferences_parent_class)->finalize (object);
}

/*
 * boilerplate_preferences_init:
 * @self: An instance of #BoilerplatePreferences
 *
 * Initializes various aspects of the window. Anything the window would need
 * before becoming presented should be done here.
 */
static void
boilerplate_preferences_init (BoilerplatePreferences *self)
{
    BoilerplatePreferencesPrivate *priv = boilerplate_preferences_get_instance_private (self);

    gtk_widget_init_template (GTK_WIDGET (self));

     /* Set window title */
    gtk_window_set_title (GTK_WINDOW (self), _("Preferences"));

    /* Set default window decorations (a.k.a GtkHeaderBar) */
    gtk_window_set_titlebar (GTK_WINDOW (self), priv->headerbar);
    gtk_container_add (GTK_CONTAINER (priv->headerbar), priv->toolbar);

    /* Connect escape key */
    g_signal_connect (G_OBJECT (self),
                      "key_press_event",
                      G_CALLBACK (_on_key_press),
                      self);

    /* Bind settings from GUI */
    priv->settings = g_settings_new (PACKAGE_ID);
/*
    g_settings_bind (priv->settings, "example-button",
                     priv->example_button, "active",
                     G_SETTINGS_BIND_DEFAULT);

    g_settings_bind(priv->settings, "example-combo",
                    priv->example_combo, "active-id",
                    G_SETTINGS_BIND_DEFAULT);
*/
}

/*
 * boilerplate_preferences_class_init:
 * @klass: An instance of #BoilerplatePreferencesClass
 *
 * Initializes the various virtural functions in #BoilerplatePreferencesClass.
 * In addtition it connects the various #GtkWidgets from the gresource file
 * using gtk_widget_class_set_template_from_resource().
 */
static void
boilerplate_preferences_class_init (BoilerplatePreferencesClass *klass)
{
    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->finalize = boilerplate_preferences_finalize;

    gtk_widget_class_set_template_from_resource (widget_class,
                                                 PACKAGE_RESOURCES "/ui/preferences.ui");

    gtk_widget_class_bind_template_child_private (widget_class, BoilerplatePreferences, toolbar);
    gtk_widget_class_bind_template_child_private (widget_class, BoilerplatePreferences, headerbar);
    gtk_widget_class_bind_template_child_private (widget_class, BoilerplatePreferences, content);
    gtk_widget_class_bind_template_child_private (widget_class, BoilerplatePreferences, pages);
    gtk_widget_class_bind_template_child_private (widget_class, BoilerplatePreferences, sidebar);

    //gtk_widget_class_bind_template_child_private (widget_class, BoilerplatePreferences, example_button);
    //gtk_widget_class_bind_template_child_private (widget_class, BoilerplatePreferences, example_combo);
}

/**
 * boilerplate_preferences_new:
 * @window: A #GtkApplicationWindow instance
 *
 * Creates a new #BoilerplatePreferences, a sub-object of #GtkWindow.
 *
 * ## Usage
 * |[
 * BoilerplatePreferences *preferences;
 *
 * preferences = boilerplate_preferences_new (window);
 * ]|
 *
 * When creating a boilerplate_preferences_new() a #BoilerplateWindow will need
 * to be passed. This is because #BoilerplatePreferences acts as a modal window
 * to the master window #BoilerplateWindow attached to the
 * #BoilerplateApplication.
 *
 * Returns: #BoilerplatePreferences; a sub-object of #GtkWindow
 */
BoilerplatePreferences *
boilerplate_preferences_new (BoilerplateWindow *window)
{
    return g_object_new (BOILERPLATE_TYPE_PREFERENCES,
                         "transient-for", window,
                         NULL);
}
